package alfred.web.core

import com.fasterxml.jackson.annotation.JsonIgnore
import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Value
import org.springframework.http.HttpStatus
import org.springframework.stereotype.Service
import org.springframework.web.bind.annotation.ResponseStatus
import java.lang.RuntimeException
import java.nio.file.Files
import java.nio.file.Paths
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.*

@Service
class Builds(
    @Value("\${ALFRED_HOME}")
    private val home: String
) {

    private val logger: Logger = LoggerFactory.getLogger(this::class.java)

    init {
        logger.info("ALFRED_HOME is $home")
    }

    fun buildConfig(build: BuildId): BuildConfig {
        val path = Paths.get(home, "builds", "$build.properties")

        if (!path.toFile().exists()) {
            throw UnknownBuild(build)
        }

        val props = Properties()
        props.load(path.toFile().inputStream())

        val env = props
            .entries
            .filter { it.key.toString() == it.key.toString().uppercase() }
            .associate { Pair(it.key.toString(), it.value.toString()) }

        // diverge into ScriptedBasedConfig and GitBasedConfig
        return BuildConfig(
            user = props.getProperty("user"),
            gitWorkspace = props.getProperty("workspace") ?: "",
            apikey = props.getProperty("apikey") ?: "",
            script = props.getProperty("script"),
            gitRepo = props.getProperty("git.repo.url"),
            gitCloneTimeout = props.getProperty("git.close.timeout")?.toLong() ?: 30L,
            env = env
        )
    }

    fun createLogFile(build: BuildId): LogFile {
        val nowStr = LocalDateTime.now().format(DateTimeFormatter.ISO_DATE_TIME)
        val fileName = "$build.$nowStr.log"

        val path = Paths.get(home, "logs", fileName)
        path.toFile().parentFile.mkdirs()

        return LogFile(Files.createFile(path).toFile())
    }

}

@ResponseStatus(value = HttpStatus.NOT_FOUND, reason = "Unknown Build Id")
class UnknownBuild(val build: BuildId) : RuntimeException() {
    override fun toString() =
        "unknown build: $build"
}

data class BuildConfig(
    val user: String,
    @field:JsonIgnore
    val apikey: String,
    val script: String?,
    val gitRepo: String?,
    val gitCloneTimeout: Long,
    val gitWorkspace: String,
    @field:JsonIgnore
    val env: Map<String, String>
)

typealias BuildId = String
