package alfred.web.core

import org.slf4j.Logger
import org.slf4j.LoggerFactory
import org.springframework.stereotype.Service
import java.nio.file.Path
import java.util.concurrent.TimeUnit

@Service
class GitRunner(
    val builds: Builds,
    val handles: Handles,
    val workspaces: Workspaces
) {

    val scriptsDir = ".alfred"

    fun run(buildId: BuildId, rev: String): ProcessInfo {
        val config = builds.buildConfig(buildId)
        val logFile = builds.createLogFile(buildId)

        logger.info("preparing process for build $buildId with config $config")
        logger.info("log file: $logFile")

        val ctx = BuildContext(
            buildId = buildId,
            config = config,
            logFile = logFile,
            rev = rev
        )

        Thread {
            workspaces.withWorkspace(ctx) { wsPath ->
                clone(ctx, wsPath)
                execScripts(ctx, wsPath)
            }
        }.start()

        return ProcessInfo(-1, logFile)
    }

    private fun clone(ctx: BuildContext, wsPath: Path) {
        logger.info("build ${ctx.buildId}: cloning ${ctx.config.gitRepo} into $wsPath")

        val process = processBuilder(ctx.config, ctx.logFile, "")
            .command("git", "clone", ctx.config.gitRepo, ".")
            .directory(wsPath.toFile())
            .start()
        handles.add(Handle(process.toHandle(), ctx.buildId))
        val cloneSuccess = process.waitFor(ctx.config.gitCloneTimeout, TimeUnit.SECONDS)

        logger.info("build ${ctx.buildId}: checkout rev ${ctx.rev}")
        processBuilder(ctx.config, ctx.logFile, "")
            .command("git", "checkout", ctx.rev)
            .directory(wsPath.toFile())
            .start()
            .waitFor()

        if (!cloneSuccess) {
            throw FailedToClone(ctx.buildId, ctx.config.gitRepo ?: "[no repo configured]")
        }
    }

    private fun execScripts(ctx: BuildContext, wsPath: Path) {
        val scriptFiles = listOf("pre.sh", "job.sh", "post.sh")

        logger.info("build ${ctx.buildId}: looking for scripts $scriptFiles in $scriptsDir/")

        scriptFiles.forEach { script ->
            val scriptFile = shFile(wsPath, script)
            if (scriptFile.exists()) {
                logger.info("build ${ctx.buildId}: found script $script, running it")

                ctx.logFile.append("\nRunning build file: $script\n")

                val scriptProcess = processBuilder(ctx.config, ctx.logFile, ctx.rev)
                    .command(scriptFile.absolutePath)
                    .directory(wsPath.toFile())
                    .start()
                handles.add(Handle(scriptProcess.toHandle(), ctx.buildId))

                val ret = scriptProcess.waitFor()
                logger.info("build ${ctx.buildId}: script $script returned $ret")

                ctx.logFile.append("\n$script returned $ret\n")
            } else {
                ctx.logFile.append("\nBuild file $scriptsDir/$script not found\n")
            }
        }
    }

    private fun shFile(wsPath: Path, name: String) =
        wsPath.resolve(scriptsDir).resolve(name).toFile()

    val logger: Logger = LoggerFactory.getLogger(this::class.java)

}

data class BuildContext(
    val config: BuildConfig,
    val logFile: LogFile,
    val rev: String,
    val buildId: BuildId
)

class FailedToClone(
    private val buildId: BuildId,
    private val gitRepo: String
) : RuntimeException() {
    override fun toString() =
        "${this::class}: failed to clone $gitRepo for build id $buildId"
}
